﻿/// Opacity Render Optimization
// Written by Koby Goldberg - AUG 2008
//  Update - v.2 : NOV 2008
//
// What the script does:
//======================
// This script works on all AV layers in the current composition
// and trims the in-point and out-point of each layer such that,
// the in-point is the first frame in the timeline where the layer opacity is greater than 0.
// and the out-point is the last frame in the timeline where the layer opacity is greater than 0.
// if in the middle of that area the opacity would drop to zero, the script will split the layer to 2 layers
// and each copy will be treated as explained above.
//  New Feature (Nov 22): Locked layers will be discarded.
//
//
// The goal of the script:
//========================
// This script takes into count opacity expressions and keyframes, 
// and by eliminating the areas where the layer is fully transparent (opacity=0%)
// it actualy saves After Effects from unnecesarry rendering.
// (It might seem strange that AE doesn't do that automatically, but it doesn't. 
// Otherwise I wouldn't have written this script...
// AE calculates all layers, even if they are fully transparent)
//
//
// When to use this script:
//=========================
// The greatest advantage of this script is achieved when it is used on animations that are controlled by expressions,
// with LOTS of layers but only a few of them are visible in any given time.
// By adding an expression to the opacity, that would make certain that the opacity is 0% when the layer is out of frame, 
// (or from some other reason should not be visible) and is 100% when the layer should be seen, you will enable script to achieve his goal.
// (you could use keyframes too, instead of expressions, but I think that would be too much woek)
// The script will trim the layers based on the opacity values, and thus will save a lot of rendering time!
//
//

{
// create an undo group 
app.beginUndoGroup("Opacity Render Optimization");

var curItem = app.project.activeItem; 
var T = curItem.frameDuration;

// check if comp is selected 
if (curItem == null || !(curItem instanceof CompItem)){ 

// if no comp selected, display an alert
alert("Please establish a comp as the active item and run the script again"); 

} else { 
// otherwise, loop through each layer in the selected comp
for (var b = 1; b <= curItem.numLayers; ++b){
	// define the layer in the loop we're currently looking at
	var curLayer = curItem.layer(b);
	// check if that layer is a footage layer
	if (curLayer.locked==true)
		continue;
	if (curLayer.matchName == "ADBE AV Layer"){
		var state = 0;
		var outpoint = curLayer.outPoint;
		for (var t=curLayer.inPoint; t<=curLayer.outPoint; t+=T){
			if (state==0 && curLayer.opacity.valueAtTime(t,false)>0)
				state = 1;
			if (state==1 && curLayer.opacity.valueAtTime(t,false)==0)
				state = 2;
			if (state==2 && curLayer.opacity.valueAtTime(t,false)>0){
				state = 1;
				curLayer.outPoint = t - T;
				curLayer.duplicate();
				curLayer.inPoint = t - T;
				curLayer.outPoint = outpoint;
				break;
			}
		}
	}
}
for (var b = 1; b <= curItem.numLayers; ++b){
// define the layer in the loop we're currently looking at
curLayer = curItem.layer(b);
if (curLayer.locked==true)
	continue;
// check if that layer is a footage layer
if (curLayer.matchName == "ADBE AV Layer"){

// serch for inPoint:
var start = curLayer.inPoint;

for (var t=curLayer.inPoint-T; t<=curLayer.outPoint; t+=T)
{
   if (curLayer.opacity.valueAtTime(t,false)>0)
	{
		start = t-2*T;
		break;
	}
}
// serch for outPoint:
var stop = 0;
for (var t=Math.floor(curLayer.outPoint/T)*T; t>=0; t-=T)
{
   if (curLayer.opacity.valueAtTime(t,false)>0)
   {
      stop = t+2*T;
      break;
   }
}
curLayer.inPoint = start;
curLayer.outPoint = stop;
}
}
}

// close the undo group
app.endUndoGroup(); 
}